package com.mobai.iotDB.service.impl;

import com.mobai.domain.Vehicle;
import com.mobai.iotDB.config.IotDBSessionConfig;
import com.mobai.iotDB.service.IotDbServer;
import com.mobai.req.VehicleReq;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.iotdb.session.SessionDataSet;
import org.apache.iotdb.tsfile.read.common.Field;
import org.apache.iotdb.tsfile.read.common.RowRecord;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.rmi.ServerException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * description: iot服务实现类
 * date: 2022/8/15 9:43
 * author: zhouhong
 */

@Log4j2
@Service
public class IotDbServerImpl implements IotDbServer {

    @Resource
    private IotDBSessionConfig iotDBSessionConfig;


    @Override
    public void insertData(Map<String, String> map) throws StatementExecutionException, ServerException, IoTDBConnectionException {
        // iotDbParam: 模拟设备上报消息
        // bizkey: 业务唯一key  PK :产品唯一编码   SN:设备唯一编码
        String deviceId = "root.vin.map." + map.get("vin");
        // 将设备上报的数据存入数据库(时序数据库)
        iotDBSessionConfig.insertRecord(deviceId, Long.valueOf(map.get("drivingRoute")),
                map.keySet().stream().toList(), map.values().stream().toList());
    }


    /**
     * 查询时间区间数据并返回
     *
     * @param req
     * @return
     * @throws Exception
     */
    @Override
    public List<Map<String, String>> queryDataFromIotDb(VehicleReq req) throws Exception {
        List<Map<String, String>> mapList = new ArrayList<>();


        if (null != req.getVin()) {

            String sql = "select * from " + "root.vin.map." + req.getVin();
            if (req.getCode() != null) {
                sql = sql.replace("*", req.getCode());                    //1719641179000
            }
            // 开始时间
            if (req.getStartTime() != null && req.getStartTime() != 0) {
                sql += " where drivingRoute >= " + req.getStartTime();
            }
            // 结束时间
            if (req.getEndTime() != null && req.getEndTime() != 0) {
                sql += " and drivingRoute <= " + req.getEndTime();
            }
            SessionDataSet sessionDataSet = iotDBSessionConfig.query(sql);
            List<String> columnNames = sessionDataSet.getColumnNames();
            List<String> titleList = new ArrayList<>();
            // 排除Time字段 -- 方便后面后面拼装数据
            for (int i = 1; i < columnNames.size(); i++) {
                String[] temp = columnNames.get(i).split("\\.");
                titleList.add(temp[temp.length - 1]);
            }
            // 封装处理数据
            packagingData(req, mapList, sessionDataSet, titleList);
        } else {
            log.info("VIN不能为空！！");
        }
        return mapList;
    }

    /**
     * 封装处理数据
     *
     * @param req
     * @param mapList
     * @param sessionDataSet
     * @param titleList
     * @throws StatementExecutionException
     * @throws IoTDBConnectionException
     */
    private void packagingData(VehicleReq req, List<Map<String, String>> mapList, SessionDataSet sessionDataSet, List<String> titleList)
            throws StatementExecutionException, IoTDBConnectionException {
        int fetchSize = sessionDataSet.getFetchSize();
        if (fetchSize > 0) {
            while (sessionDataSet.hasNext()) {
                Vehicle vehicle = new Vehicle();
                RowRecord next = sessionDataSet.next();
                List<Field> fields = next.getFields();
                vehicle.setStartTime(next.getTimestamp());
                Map<String, String> map = new HashMap<>();
                for (int i = 0; i < fields.size(); i++) {

                    Field field = fields.get(i);
                    // 这里的需要按照类型获取
                    Object obj = field.getObjectValue(field.getDataType());
                    map.put(titleList.get(i), obj == null ? "null" : obj.toString());
                }
                mapList.add(map);
            }
        }
    }
}
