package com.mobai.kafka.listener;

import com.alibaba.fastjson2.JSON;
import com.mobai.domain.MqttServerModel;
import com.mobai.domain.Vehicle;
import com.mobai.domain.VehicleEvent;
import com.mobai.forest.ForestGet;
import com.mobai.utils.RedisService;
import com.mobai.vehicle.event.service.EventsService;
import com.mobai.vehicle.HandlerHelper;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.consumer.*;
import org.apache.kafka.common.TopicPartition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author Saisai
 * @className VinConsumerRunner
 * @description 描述
 * @date 2024/6/18 17:18
 */

@Log4j2
@Component
public class VinConsumerRunner implements ApplicationRunner {

    @Autowired
    private ForestGet forestGet;

    @Autowired
    private KafkaConsumerListenerExample kafkaConsumerListenerExample;

    @Autowired
    private RedisService redisService;

    @Autowired
    private EventsService eventsService;


    private final AtomicInteger start = new AtomicInteger();
    @Override
    public void run(ApplicationArguments args) throws Exception {
        if (start.get() != 0) {
            return;
        }
        synchronized (this) {
            if (start.get() != 0) {
                return;
            }
            start.set(1);

            // kafka分区监听器
            new Thread(() -> {
                ArrayList<TopicPartition> topicPartitions = new ArrayList<>();
                List<String> topics = forestGet.getIps().getData().stream().map(MqttServerModel::getTopic).toList();
                for (String topic : topics) {
                    for (int i = 0; i < 8; i++) {
                        TopicPartition topicPartition = new TopicPartition(topic, i);
                        topicPartitions.add(topicPartition);
                    }
                }
                Properties props = new Properties(){{
                    put("bootstrap.servers", "localhost:9092");
                    put("auto.commit.interval.ms", "1000");
                    put("group.id", "test");
                    put("enable.auto.commit", "true");
                    put("session.timeout.ms", "30000");
                    put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
                    put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
                }};
                KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
                consumer.assign(topicPartitions);
                try {
                    while (true) {
                        ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));
                        if (!records.isEmpty()) {
                            new Thread(() -> {
                                for (TopicPartition partition : records.partitions()) {
                                    List<ConsumerRecord<String, String>> partitionRecords = records.records(partition);
                                    for (ConsumerRecord<String, String> record : partitionRecords) {
                                        log.info("Offset = {}, Key = {}, Value = {}", record.offset(), record.key(), record.value());
                                        // 获取数据并存入
                                        Map<String,String> map = JSON.parseObject(record.value(), Map.class);
                                        // 获取对应的事件
                                        VehicleEvent events = eventsService.getEvents(map.get("vin"));
//                                        log.info("执行事件：{}",events);
                                        if (events==null)
                                            return;
                                        HandlerHelper.doHandler(events, map, redisService);
                                    }
                                }
                            }).start();
                            // 提交偏移量
                            consumer.commitAsync(new OffsetCommitCallback() {

                                @Override
                                public void onComplete(Map<TopicPartition, OffsetAndMetadata> offsets, Exception e)
                                {
                                    if (e != null) {
                                        System.out.println(offsets.toString());
                                        System.out.println(e.toString());
                                    }
                                }
                            });
                        } else {
                            Thread.sleep(10);
                        }
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    log.error("Consumer was interrupted.", e);
                } finally {
                    consumer.close();
                }
            }).start();
        }
    }
}
