package com.mobai.vehicle.event.cache.caffeine;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import lombok.Data;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author Saisai
 * @className CaffeineCache
 * @description 描述
 * @date 2024/6/21 16:15
 */
@Data
@Configuration
public class CaffeineCache {

    /**
     * 5秒钟缓存
     * @return
     */
    @Bean(name = "createFiveCaffeine")
    public  Cache createFiveCaffeine() {
         return Caffeine.newBuilder()
                // 设置初始连接数
                .initialCapacity(1000)
                //5秒没有读写自动删除
                .expireAfterAccess(5, TimeUnit.SECONDS)
                //最大容量1024个，超过会自动清理空间
                .maximumSize(1024)
                .removalListener(((key, value, cause) -> {
                    //清理通知 key,value ==> 键值对   cause ==> 清理原因
                }))
                .build();
    }

    /**
     * 半小时缓存
     * @return cache缓存
     */
    @Bean(name = "createHalfCaffeine")
    public  Cache createHalfCaffeine() {
        return Caffeine.newBuilder()
                // 设置初始连接数
                .initialCapacity(1000)
                //半小时没有读自动删除 访问
                .expireAfterAccess(30, TimeUnit.MINUTES)
                //最大容量1024个，超过会自动清理空间
                .maximumSize(1024)
                .removalListener(((key, value, cause) -> {
                    //清理通知 key,value ==> 键值对   cause ==> 清理原因
                }))
                .build();
    }

/**
     * 10分钟缓存
     * @return cache缓存
     */
    @Bean(name = "createTenMinuteCaffeine")
    public  Cache createTenMinuteCaffeine() {
        return Caffeine.newBuilder()
                // 设置初始连接数
                .initialCapacity(1000)
                //半小时没有读自动删除 访问
                .expireAfterAccess(10, TimeUnit.MINUTES)
                //最大容量1024个，超过会自动清理空间
                .maximumSize(1024)
                .removalListener(((key, value, cause) -> {
                    //清理通知 key,value ==> 键值对   cause ==> 清理原因
                }))
                .build();
    }

}
