package com.mobai.vehicle.event.client.config;

import com.mobai.vehicle.event.client.domain.KafkaConfig;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

/**
 * @author Mobai
 * @className MsgComponent
 * @description 描述
 * @date 2024/6/14 21:47
 */

@Log4j2
@Component
public class MsgComponent {

    /**
     * 队列声明
     *
     * @param kafkaConfig
     * @return
     */
    @Bean
    public Queue initVehicleEventClientQueue(KafkaConfig kafkaConfig) {
        return new Queue(kafkaConfig.queueName(), true, true, true);
    }


    /**
     * 绑定交换机和队列
     *
     * @param vehicleEventExchange        交换机
     * @param initVehicleEventClientQueue 绑定队列
     * @return 绑定结果
     */
    @Bean
    public Binding binding(FanoutExchange vehicleEventExchange,
                           Queue initVehicleEventClientQueue) {
        return BindingBuilder.bind(initVehicleEventClientQueue).to(vehicleEventExchange);
    }


    @Bean
    public SimpleMessageListenerContainer messageListenerContainer(
            ConnectionFactory connectionFactory,
            KafkaConfig kafkaConfig) {
        SimpleMessageListenerContainer simpleMessageListenerContainer = new SimpleMessageListenerContainer(connectionFactory);
        //针对哪些队列（参数为可变参数）
        simpleMessageListenerContainer.setQueueNames(kafkaConfig.queueName());
        //同时有多少个消费者线程在消费这个队列，相当于线程池的线程数字。
        simpleMessageListenerContainer.setConcurrentConsumers(3);
        //最大的消费者线程数
        simpleMessageListenerContainer.setMaxConcurrentConsumers(5);
        //设置消息确认方式    NONE=不确认,MANUAL=手动确认,AUTO=自动确认;
        //自动确认
        simpleMessageListenerContainer.setAcknowledgeMode(AcknowledgeMode.AUTO);
        //simpleMessageListenerContainer.setMessageListener(message -> log.info("springboot.rabbitmq-queue接收到的消息：[{}]", message));
        //手动确认(单条确认)
        simpleMessageListenerContainer.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        simpleMessageListenerContainer.setMessageListener(
                (ChannelAwareMessageListener) (message, channel) -> {
                    log.info("springboot.rabbitmq-queue接收到的消息：[{}]", message);
                    if (channel != null) {
                        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                    }
                });
        //消费端限流
        simpleMessageListenerContainer.setPrefetchCount(1);
        return simpleMessageListenerContainer;
    }


}
