package com.mobai.vehicle.event.factory;

import com.alibaba.fastjson2.JSON;
import com.github.benmanes.caffeine.cache.Cache;
import com.mobai.bugMalfuntion.BugMalfunctionResult;
import com.mobai.bugMalfuntion.FaultCode;
import com.mobai.domain.Vehicle;
import com.mobai.utils.RedisService;
import com.mobai.vehicle.EventActive;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 故障处理工厂
 *
 * @author Saisai
 * @className BugMalfunctionFactory
 * @description 描述
 * @date 2024/6/20 14:42
 */
@Log4j2
@Service
public class BugMalfunctionFactory implements EventActive {
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Resource(name = "createFiveCaffeine")
    private Cache cacheFive;


    private BugMalfunctionFactory() {
    }

    public static BugMalfunctionFactory getInstance() {
        return new BugMalfunctionFactory();
    }

    @Override
    public void activeEvent(Map<String,String> vehicle, RedisService redisService) {
        BugMalfunctionResult result = new BugMalfunctionResult();
        result.setVin(vehicle.get("vin"));
        result.setTimestamp(Long.valueOf(vehicle.get("drivingRoute")));
        if (vehicle.get("vehicleStatus")!=null&&vehicle.get("vehicleStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.VEHICLE.getString());
            this.active(result);
        }
        if (vehicle.get("chargingStatus")!=null&&vehicle.get("chargingStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.CHARGING.getString());
            this.active(result);
        }
        if (vehicle.get("operatingStatus")!=null&&vehicle.get("operatingStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.OPERATING.getString());
            this.active(result);
        }
        if (vehicle.get("socStatus")!=null&&vehicle.get("socStatus").equals(""+0)) {
            result.setFaultCode(FaultCode.SOC.getString());
            this.active(result);
        }
        if (vehicle.get("chargingEnergyStorageStatus")!=null&&vehicle.get("chargingEnergyStorageStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.CHARGING_ENERGY.getString());
            this.active(result);
        }
        if (vehicle.get("driveMotorStatus")!=null&&vehicle.get("driveMotorStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.DRIVE_MOTOR.getString());
            this.active(result);
        }
        if (vehicle.get("positionStatus")!=null&&vehicle.get("positionStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.POSITION.getString());
            this.active(result);
        }
        if (vehicle.get("easStatus")!=null&&vehicle.get("easStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.EAS.getString());
            this.active(result);
        }
        if (vehicle.get("ptcStatus")!=null&&vehicle.get("ptcStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.PTC.getString());
            this.active(result);
        }
        if (vehicle.get("epsStatus")!=null&&vehicle.get("epsStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.EPS.getString());
            this.active(result);
        }
        if (vehicle.get("absStatus")!=null&&vehicle.get("absStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.ABS.getString());
            this.active(result);
        }
        if (vehicle.get("mcuStatus")!=null&&vehicle.get("mcuStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.MCU.getString());
            this.active(result);
        }
        if (vehicle.get("heatingStatus")!=null&&vehicle.get("heatingStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.HEATING.getString());
            this.active(result);
        }
        if (vehicle.get("batteryStatus")!=null&&vehicle.get("batteryStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.BATTERY.getString());
            this.active(result);
        }
        if (vehicle.get("batteryInsulationStatus")!=null&&vehicle.get("batteryInsulationStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.BATTERY_INSULATION.getString());
            this.active(result);
        }
        if (vehicle.get("dcdcStatus")!=null&&vehicle.get("dcdcStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.DCDC.getString());
            this.active(result);
        }
        if (vehicle.get("chgStatus")!=null&&vehicle.get("chgStatus").equals(0+"")) {
            result.setFaultCode(FaultCode.CHG.getString());
            this.active(result);
        }
    }


    /**
     * 发布异常开始事件
     *
     * @param result
     */
    public void sendBugMalfunction(BugMalfunctionResult result) {
        log.info("发布异常事件");
        rabbitTemplate.convertAndSend("zhiLian-vehicle-start", JSON.toJSONString(result));
    }

    /**
     * 处理异常信息
     */
    public void active(BugMalfunctionResult result) {
        // 5秒验证缓存
        // 拼接异常信息
        String vinInfo = result.getVin() + "-" + result.getFaultCode();
        Object ifPresent = cacheFive.getIfPresent(vinInfo);
        if (null == ifPresent) {
            log.error("异常开始");
            log.info(vinInfo);
            redisTemplate.opsForValue().set(vinInfo, 0 + "", 10L, TimeUnit.SECONDS);
            sendBugMalfunction(result);
        }
//        else {
        //存入缓存
        cacheFive.put(vinInfo, 0 + "");
        log.warn("异常中·······  原因：{}",result.getFaultCode());
        redisTemplate.expire(vinInfo, 10, TimeUnit.SECONDS);
//        }

    }
}
