package com.mobai.vehicle.event.factory;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.mobai.iotDB.service.impl.IotDbServerImpl;
import com.mobai.req.VehicleReq;
import com.mobai.resp.RespResult;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static java.util.Arrays.stream;

/**
 * 指标预警工厂
 *
 * @author Saisai
 * @className BugMalfunctionFactory
 * @description 描述
 * @date 2024/6/20 14:42
 */


@Data
@Log4j2
@Component
public class StandardWarningFactory {


    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    /**
     * keycode-level-times
     */
    @Resource(name = "createTenMinuteCaffeine")
    private Cache<String, String> cache;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private IotDbServerImpl iotDbServerImpl;

    private Map<String, ScheduledExecutorService> scheduleMap;

    /**
     * keycode-opportunity-level
     */
    private ConcurrentHashMap<String, List<String>> vinLevel;

    private static List<String> vins;

    static {
        vins = new ArrayList<String>();
    }

//    @RabbitListener(bindings = @QueueBinding(
//            value = @Queue,
//            exchange = @Exchange(name = "vehicle-prop", type = ExchangeTypes.FANOUT)
//    ))
    @RabbitListener(queues = "zhiLian-vehicle-index")
    public void changeProp(String vin) {
        vin= vin.replaceAll("\"","");
        log.info("更新指标缓存");
        List<String> attrs = new ArrayList<>();
        List<String> propList = redisTemplate.opsForList().range(vin + "Index", 0, -1);
        for (String string : propList) {
            JSONObject prop = JSON.parseObject(string);
            // 周期延迟时长
            attrs.add(prop.get("keyCode") + "-" + prop.get("opportunity") + "-" + prop.get("attributeValue"));
        }
        cache.put(vin + "-keyCode", JSON.toJSONString(attrs)); // vin    keycode-level-attributeValue
        log.info("更新缓存：{}", vin);
        // 指标未编写和存入
    }


    /**
     * 上线开启事件的车辆
     *
     * @param vin
     */
    @RabbitListener(queues = "standard-Warn-Event-Start")
    public void joinStandardVehicle(String vin) {
        List<String> cacheProps = new ArrayList<>();
        log.info("车辆上线：{}", vin);
        List<String> props = redisTemplate.opsForList().range(vin + "Index", 0, -1);
        redisTemplate.expire(vin + "Index", 10, TimeUnit.MINUTES);
        props.forEach(prop -> {
            log.info("缓存存入操作");
            JSONObject jsonObject = JSON.parseObject(prop);
            String keyCode = String.valueOf(jsonObject.get("keyCode"));
            String level = String.valueOf(jsonObject.get("attributeValue"));
            String opportunity = String.valueOf(jsonObject.get("opportunity"));
            cacheProps.add(keyCode + "-" + opportunity + "-" + level);
        });
        new Thread(() -> {
            cache.put(vin + "-keyCode", JSON.toJSONString(cacheProps));
            vins.add(vin);
            this.doActive(vin);
        }).start();
        // 存入车辆指标缓存 code-opportunity-level

    }

    private void doActive(String vin) {
        // 每次都执行
        List<String> strings = JSON.parseArray(cache.getIfPresent(vin + "-keyCode"), String.class);
        if (strings == null || strings.size() == 0) {
            changeProp(vin);
        }
        strings = JSON.parseArray(cache.getIfPresent(vin + "-keyCode"), String.class);
        Timer timer = new Timer();
        if (vins == null) {
            vins = new ArrayList<>();
        }
        List<String> finalStrings = strings;
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                new Thread(() -> {
                    int first = 0;
                for (String string : finalStrings) {
                        avtiveThread(vin, string);
                        if (first==0) doActive(vin);
                        first = 1;
                }
                }).start();
            }
        };
        timer.schedule(timerTask, Long.parseLong(strings.get(0).split("-")[1]) * 1000);

        if (!vins.contains(vin)) {
            timer.cancel();
            return;
        }
    }

    private void avtiveThread(String vin, String string) {
        String[] split = string.split("-");
        try {
            List<String> list = new ArrayList<>();
            List<Map<String, String>> mapList = iotDbServerImpl.queryDataFromIotDb(new VehicleReq() {{
                setVin(vin);
                setCode(split[0]);
                setEndTime(new Date().getTime());
                setStartTime(new Date().getTime() - Long.parseLong(split[1]) * 1000);
            }});
//            mapList.stream().map(map -> list.addAll(map.values()));
            for (Map<String, String> stringMap : mapList) {
                list.addAll(stringMap.values());
            }
            if (list.size() == 0) {
                return;
            }
            List<Double> collect = new ArrayList<>();
            for (String s : list) {
                BigDecimal bigDecimal = new BigDecimal(s);
                collect.add(bigDecimal.doubleValue());
            }
            Double avg = (Double) collect.stream().mapToDouble(number -> number).average().getAsDouble();
            List<Double> result = new ArrayList<>();
            for (Double i : collect) {
                result.add(i - avg);
                if (i > Integer.parseInt(split[2])) {
                    rabbitTemplate.convertAndSend("standardWarning-error", vin+"-"+split[0] + ":" + i);
                }
            }
            RespResult resp = new RespResult() {{
                setKeyCode(split[0]);
                setVin(vin);
                setResList(result);
                setStartTime(new Date().getTime() - Long.parseLong(split[1]) * 1000);
                setStartTime(new Date().getTime());
            }};
            System.out.println("当前指标为："+split[2]);
            log.info("执行指标预警、滑窗结果滑窗结果：{}", result);
            System.out.println(resp);
            rabbitTemplate.convertAndSend("standardWarning-result", JSON.toJSONString(resp));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }


    /**
     * 下线关闭事件的车辆
     *
     * @param vin
     */
    @RabbitListener(queues = "standard-Warn-Event-End")
    public void outStandardVehicle(String vin) {
        log.info("车辆下线：{}", vin);
        if (vins != null) {
            if (vins.contains(vin))
                vins.remove("vin");
        }
//        rabbitTemplate.convertAndSend();
    }

}
