package com.mobai.vehicle.event.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.benmanes.caffeine.cache.Cache;
import com.mobai.domain.EventReq;
import com.mobai.domain.Result;
import com.mobai.domain.VehicleEvent;
import com.mobai.vehicle.event.mapper.VehicleEventMapper;
import com.mobai.vehicle.event.service.EventActiveService;
import com.mobai.vehicle.event.service.EventsService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 事件处理
 *
 * @author Saisai
 * @className EventsServiceImpl
 * @description 描述
 * @date 2024/6/18 11:39
 */

@Log4j2
@Service
public class EventsActiveServiceImpl extends ServiceImpl<VehicleEventMapper, VehicleEvent> implements EventsService {

    @Autowired
    private EventActiveService eventActiveService;

    @Resource(name = "createHalfCaffeine")
    private Cache cacheHalf;

    @Override
    public Result activeEvent(EventReq req) {
        switch (req.getEventType()) {
            case 0:
                return eventActiveService.realTimeData(req);
        }
        return null;
    }


    /**
     * 通过车辆vin获取故障事件信息及缓存数据
     *
     * @param vin
     * @return vin + events
     */
    public VehicleEvent getEvents(String vin) {
        // 使用caffeine缓存
        Object vehicleEvent = cacheHalf.getIfPresent(vin + "-events");
        if (vehicleEvent != null) {
            // 若有访问更新缓存时间
            cacheHalf.put(vin + "-events", vehicleEvent);
            return (VehicleEvent) vehicleEvent;
        }
        vehicleEvent = this.getOne(new LambdaQueryWrapper<>() {{
            eq(VehicleEvent::getVin, vin);
        }});
        if (vehicleEvent==null){
            log.warn("该车辆没有其他事件");
            return null;
        }
        cacheHalf.put(vin + "-events", vehicleEvent);
        log.info("数据库");
        Object ifPresent = cacheHalf.getIfPresent(vin + "-events");
        log.info("缓存信息为：{}", ifPresent);
        return (VehicleEvent) vehicleEvent;
    }

}
